@@ -6,6 +6,8 @@ pub struct Theme {
     pub border: Color,
     pub selected_foreground: Color,
     pub selected_background: Color,
+    pub marked_foreground: Color,
+    pub marked_background: Color,
     pub found: Color,
     pub found_selected_background: Color,
     pub status: Color,
@@ -28,6 +30,8 @@ impl Theme {
             border: gutter,
             selected_foreground: Color::Rgb(192, 192, 192),
             selected_background: Color::Rgb(62, 61, 50),
+            marked_foreground: Color::Rgb(220, 230, 255),
+            marked_background: Color::Rgb(40, 50, 80),
             found: Color::Rgb(200, 0, 0),
             found_selected_background: Color::LightYellow,
             status: gutter,
@@ -48,6 +52,8 @@ impl Theme {
             border: gutter,
             selected_foreground: Color::Rgb(73, 72, 62),
             selected_background: Color::Rgb(230, 227, 196),
+            marked_foreground: Color::Rgb(0, 40, 80),
+            marked_background: Color::Rgb(220, 235, 255),
             found: Color::Rgb(200, 0, 0),
             found_selected_background: Color::LightYellow,
             status: gutter,
