@@ -6,6 +6,7 @@ use crate::input::Control;
 use crate::sort::{SortOrder, Sorter};
 
 use std::cmp::min;
+use std::collections::HashSet;
 use std::sync::Arc;
 use std::time::{Duration, Instant};
 
@@ -27,6 +28,11 @@ impl RowsFilter {
     }
 }
 
+pub struct MarkToggleResult {
+    pub record_num: usize,
+    pub marked: bool,
+}
+
 #[derive(Clone)]
 pub struct SelectionDimension {
     index: Option<u64>,
@@ -260,6 +266,7 @@ pub struct RowsView {
     sort_order: SortOrder,
     pub selection: Selection,
     perf_stats: Option<PerfStats>,
+    marked_rows: HashSet<usize>,
 }
 
 impl RowsView {
@@ -281,6 +288,7 @@ impl RowsView {
             sort_order: SortOrder::Ascending,
             selection: Selection::default(num_rows),
             perf_stats: None,
+            marked_rows: HashSet::new(),
         };
         Ok(view)
     }
@@ -297,6 +305,10 @@ impl RowsView {
         &self.rows
     }
 
+    pub fn marked_rows(&self) -> &HashSet<usize> {
+        &self.marked_rows
+    }
+
     pub fn get_column_name_from_global_index(&self, column_index: usize) -> String {
         self.raw_headers()
             .get(column_index)
@@ -374,6 +386,21 @@ impl RowsView {
         self.selection.row.set_bound(num_rows_rendered);
     }
 
+    pub fn set_reader(
+        &mut self,
+        reader: CsvLensReader,
+        filter_finder: Option<&find::Finder>,
+    ) -> CsvlensResult<()> {
+        self.reader = reader;
+        self.headers = Self::get_default_headers_from_reader(&self.reader);
+        if let Some(finder) = filter_finder {
+            self.set_filter(finder)?;
+        } else {
+            self.do_get_rows()?;
+        }
+        Ok(())
+    }
+
     pub fn set_filter(&mut self, finder: &find::Finder) -> CsvlensResult<()> {
         let filter = RowsFilter::new(finder, self.rows_from, self.num_rows);
         // only need to reload rows if the currently shown indices changed
@@ -397,12 +424,31 @@ impl RowsView {
         self.filter.is_some()
     }
 
-    pub fn reset_filter(&mut self) -> CsvlensResult<()> {
-        if !self.is_filter() {
-            return Ok(());
+    pub fn reset_filter(&mut self, preserve_row_selection: bool) -> CsvlensResult<()> {
+        if let Some(filter) = &self.filter {
+            let mut record_index_to_preserve = None;
+            if preserve_row_selection {
+                if let Some(row_selection_index) = self.selection.row.index {
+                    record_index_to_preserve =
+                        filter.indices.get(row_selection_index as usize).cloned();
+                } else {
+                    record_index_to_preserve = filter.indices.first().cloned();
+                }
+            }
+            if let (Some(sorter), Some(index)) = (&self.sorter, record_index_to_preserve) {
+                // The row index is in terms of original data, but the view is now sorted. Need to
+                // get the actual row number to scroll to that points to that record.
+                record_index_to_preserve = sorter.get_record_order(index, self.sort_order);
+            }
+            self.filter = None;
+            if let Some(n) = record_index_to_preserve {
+                self.handle_scroll_to((n as usize).saturating_add(1))
+            } else {
+                self.do_get_rows()
+            }
+        } else {
+            Ok(())
         }
-        self.filter = None;
-        self.do_get_rows()
     }
 
     pub fn columns_filter(&self) -> Option<&Arc<ColumnsFilter>> {
@@ -523,7 +569,7 @@ impl RowsView {
     }
 
     pub fn get_total_line_numbers_approx(&self) -> Option<usize> {
-        self.reader.get_last_indexed_line_number()
+        Some(self.reader.get_approx_line_numbers())
     }
 
     pub fn in_view(&self, row_index: u64) -> bool {
@@ -589,23 +635,35 @@ impl RowsView {
                 self.selection.row.select_last()
             }
             Control::ScrollTo(n) => {
-                let mut rows_from = n.saturating_sub(1) as u64;
-                if let Some(n) = self.bottom_rows_from() {
-                    rows_from = min(rows_from, n);
-                }
-                self.set_rows_from(rows_from)?;
-                self.selection.row.select_first()
+                self.handle_scroll_to(*n)?;
             }
             _ => {}
         }
         Ok(())
     }
 
+    /// Scroll to a 1-based record number
+    fn handle_scroll_to(&mut self, n: usize) -> CsvlensResult<()> {
+        // Don't scroll beyond the bottom row
+        let mut rows_from = n.saturating_sub(1) as u64;
+        if let Some(n) = self.bottom_rows_from() {
+            rows_from = min(rows_from, n);
+        }
+        self.set_rows_from(rows_from)?;
+        // Set row selection to the correct row
+        if self.selection.row.index().is_some() {
+            self.selection
+                .row
+                .set_index(n.saturating_sub(1).saturating_sub(rows_from as usize) as u64);
+        }
+        Ok(())
+    }
+
     fn get_total_line_numbers_indexed(&self) -> Option<usize> {
         if let Some(max_line_number) = self
             .reader
             .get_total_line_numbers()
-            .or_else(|| self.reader.get_last_indexed_line_number())
+            .or_else(|| Some(self.reader.get_approx_line_numbers()))
         {
             if let Some(filter) = &self.filter {
                 if let Some(max_index) = filter.max_index {
@@ -651,7 +709,7 @@ impl RowsView {
         out
     }
 
-    fn do_get_rows(&mut self) -> CsvlensResult<()> {
+    pub fn do_get_rows(&mut self) -> CsvlensResult<()> {
         let start = Instant::now();
         let (mut rows, reader_stats) = if let Some(filter) = &self.filter {
             let indices = &filter.indices;
@@ -681,6 +739,28 @@ impl RowsView {
         Ok(())
     }
 
+    pub fn toggle_mark(&mut self, row_index: usize) -> Option<MarkToggleResult> {
+        let record_num = self.rows.get(row_index)?.record_num;
+
+        if self.marked_rows.remove(&record_num) {
+            return Some(MarkToggleResult {
+                record_num,
+                marked: false,
+            });
+        };
+
+        self.marked_rows.insert(record_num);
+
+        Some(MarkToggleResult {
+            record_num,
+            marked: true,
+        })
+    }
+
+    pub fn clear_marks(&mut self) {
+        self.marked_rows.clear();
+    }
+
     #[cfg(test)]
     pub fn wait_internal(&self) {
         self.reader.wait_internal()
